/* This is a script that lists the users on the BBS.
 *
 * Author: Eric Oulashin (AKA Nightfox)
 * BBS: Digital Distortion
 * BBS address: digdist.bbsindex.com
 *
 * Date       User              Description
 * 2009-05-02 Eric Oulashin     Created
 * 2009-05-03 Eric Oulashin     Added a Synchronet version check.
 * 2009-05-04 Eric Oulashin     Changed the User variable's name from user
 *                              to theUser to avoid stomping on the user
 *                              object declared in sbbsdefs.js.
 * 2009-05-07 Eric Oulashin     Updated to allow the user to sort the
 *                              list alphabetically by user alias.
 * 2009-05-10 Eric Oulashin     Removed the "donePrompt" color from the colors
 *                              array, because it wasn't being used.
 * 2009-05-13 Eric Oulashin     Version 1.01
 *                              Updated to be free of DigitalDistortionDefs.js.
 *                              Added VERSION and VER_DATE, although they aren't
 *                              displayed to the user.
 * 2009-07-23 Eric Oulashin     Version 1.02
 *                              Minor update: Commented out VERSION and VER_DATE
 *                              to avoid re-declaration problems.  These weren't
 *                              being displayed to the user anyway.  Also,
 *                              changed the special character variables to be
 *                              declared with "var" rather than "const" to avoid
 *                              re-declaration errors.
 * 2009-07-31 Eric Oulashin     Updated centerText() to get the width of the
 *                              text after using strip_ctrl() so that it
 *                              doesn't count control characters.
 * 2009-08-17 Eric Oulashin     Version 1.03
 *                              Updated to read configuration options from a
 *                              configuration file, DigitalDistortionListUsers.cfg.
 *                              Added options there for excluding the sysop,
 *                              sorting the list (true/false/ask), whether
 *                              or not to display QWK accounts, and colors.
 * 2009-08-19 Eric Oulashin     Version 1.04
 *                              Bug fix: Now skips invalid user numbers, so it
 *                              will display the complete list of users (rather
 *                              than looping through 1 to system.stats.total_users,
 *                              because user numbers aren't necessarily sequential).
 * 2010-02-17 Eric Oulashin     Version 1.05
 *                              Updated to skip inactive user slots.
 * 2011-02-13 Eric Oulashin     Version 1.06
 *                              Moved the user lister code into its own class.
 *                              Added the getHTML function for use in a web page.
 * 2011-11-23 Eric Oulashin     Version 1.07
 *                              Fixed a typo in a variable name.  Also fixed the
 *                              error text displayed when the Synchronet version
 *                              is not current enough.  Thanks to Slinky for
 *                              catching these.
 * 2011-11-25 Eric Oulashin     Version 1.08
 *                              Bug fix in DDUserLister_GetHTML(): For Synchronet 3.11+,
 *                              made lastuser equal to lastuser = system.lastuser + 1
 *                              instead of just system.lastuser.  Thanks to Slinky
 *                              for reporting this.
 * 2013-11-17 Eric Oulashin     Version 1.09
 *                              Bug fix in DDUserLister_GetHTML(): Now uses the correct
 *                              user numbers for the user information links when the list
 *                              is sorted.
 */

load("sbbsdefs.js");

// This script requires Synchronet version 3.11 or higher.
// Exit if the Synchronet version is below the minimum.
if (system.version_num < 31100)
{
	var message = "nhyi* Warning:nhw Digital Distortion List Users "
	             + "requires version g3.11w or\r\n"
	             + "higher of Synchronet.  This BBS is using version g" + system.version
	             + "w.  Please notify the sysop.";
	console.crlf();
	console.print(message);
	console.crlf();
	console.print("p");
	exit();
}

//var VERSION = "1.09";
//var VER_DATE = "2013-11-17";

// Column inner lengths
var USER_NUM_LEN = 5;
var USER_NAME_LEN = 18;
var LOCATION_LEN = 25;
var LAST_CALL_DATE_LEN = 8;
var NUM_CALLS_LEN = 7;
var CONNECTION_TYPE_LEN = 8;

// Single-line box drawing/border characters.
// Note: These are declared using "var" rather
// than "const" to avoid re-declaration problems.
var UPPER_LEFT_SINGLE = "";
var HORIZONTAL_SINGLE = "";
var UPPER_RIGHT_SINGLE = "";
var VERTICAL_SINGLE = "";
var LOWER_LEFT_SINGLE = "";
var LOWER_RIGHT_SINGLE = "";
var T_SINGLE = "";
var LEFT_T_SINGLE = "";
var RIGHT_T_SINGLE = "";
var BOTTOM_T_SINGLE = "";
var CROSS_SINGLE = "";
// Double-line box drawing/border characters.
var UPPER_LEFT_DOUBLE = "";
var HORIZONTAL_DOUBLE = "";
var UPPER_RIGHT_DOUBLE = "";
var VERTICAL_DOUBLE = "";
var LOWER_LEFT_DOUBLE = "";
var LOWER_RIGHT_DOUBLE = "";
var T_DOUBLE = "";
var LEFT_T_DOUBLE = "";
var RIGHT_T_DOUBLE = "";
var BOTTOM_T_DOUBLE = "";
var CROSS_DOUBLE = "";

///////////////////////////
// Script execution code //
///////////////////////////

// The 1st program argument specifies whether or not to execute the script code.
var executeScriptCode = true;
if ((typeof(argv[0]) != "undefined") && (argv[0] != null))
   executeScriptCode = argv[0];
// If executeScriptCode is true, then list all users.
if (executeScriptCode)
{
   var userLister = new DDUserLister();
   userLister.listAllUsers(false);
}

// End of script execution


///////////////////////////////////////////////////////////////////////////////////
// DDUserLister class stuff

// DDUserLister class constructor
function DDUserLister()
{
	this.excludeSysop = false;    // Whether or not to exclude the sysop
	this.sortUserList = false;    // Whether or not to sort the list
	this.sortType = "alpha";       // Alphabetical sort
	this.displayQWKAccts = false; // Whether or not to display QWK accounts

	// Colors array
	this.colors = new Array();
	this.colors["veryTopHeader"] = "nhw";
	this.colors["border"] = "nc";
	this.colors["colLabelText"] = "n4hw";
	this.colors["userNum"] = "hc";
	this.colors["userName"] = "hy";
	this.colors["location"] = "hm";
	this.colors["lastCallDate"] = "hr";
	this.colors["connectionType"] = "hb";
	this.colors["numCalls"] = "nhg";

	// Member methods
	this.readConfigFile = DDUserLister_ReadConfigFile;
	this.displayUserInfo = DDUserLister_DisplayUserInfo;
	this.listAllUsers = DDUserLister_ListAllusers;
	this.getHTML = DDUserLister_GetHTML;

	// Read the configuration file
	this.readConfigFile();
}
// For the DDUserLister class: Reads the configuration file and sets the various options.
function DDUserLister_ReadConfigFile()
{
   // Determine the script's startup directory.
   // This code is a trick that was created by Deuce, suggested by Rob Swindell
   // as a way to detect which directory the script was executed in.  I've
   // shortened the code a little.
   var startup_path = '.';
   try { throw dig.dist(dist); } catch(e) { startup_path = e.fileName; }
   startup_path = backslash(startup_path.replace(/[\/\\][^\/\\]*$/,''));

   // Open the configuration file
   var cfgFile = new File(startup_path + "DigitalDistortionListUsers.cfg");
   if (cfgFile.open("r"))
   {
      var settingsMode = "behavior";
      var fileLine = null;     // A line read from the file
      var equalsPos = 0;       // Position of a = in the line
      var commentPos = 0;      // Position of the start of a comment
      var setting = null;      // A setting name (string)
      var settingUpper = null; // Upper-case setting name
      var value = null;        // A value for a setting (string)
      while (!cfgFile.eof)
      {
         // Read the next line from the config file.
         fileLine = cfgFile.readln(2048);

         // fileLine should be a string, but I've seen some cases
         // where it isn't, so check its type.
         if (typeof(fileLine) != "string")
            continue;

         // If the line starts with with a semicolon (the comment
         // character) or is blank, then skip it.
         if ((fileLine.substr(0, 1) == ";") || (fileLine.length == 0))
            continue;

         // If in the "behavior" section, then set the behavior-related variables.
         if (fileLine.toUpperCase() == "[BEHAVIOR]")
         {
            settingsMode = "behavior";
            continue;
         }
         else if (fileLine.toUpperCase() == "[COLORS]")
         {
            settingsMode = "colors";
            continue;
         }

         // If the line has a semicolon anywhere in it, then remove
         // everything from the semicolon onward.
         commentPos = fileLine.indexOf(";");
         if (commentPos > -1)
            fileLine = fileLine.substr(0, commentPos);

         // Look for an equals sign, and if found, separate the line
         // into the setting name (before the =) and the value (after the
         // equals sign).
         equalsPos = fileLine.indexOf("=");
         if (equalsPos > 0)
         {
            // Read the setting & value, and trim leading & trailing spaces.
            setting = trimSpaces(fileLine.substr(0, equalsPos), true, false, true);
            settingUpper = setting.toUpperCase();
            value = trimSpaces(fileLine.substr(equalsPos+1), true, false, true).toUpperCase();

            if (settingsMode == "behavior")
            {
               // Set the appropriate setting.
               // Note: The value of SORTLIST can be true, false, or ask.
               if (settingUpper == "SORTLIST")
                  this.sortUserList = (value == "ASK" ? "ask" : (value == "TRUE"));
               else if (settingUpper == "DISPLAYQWKACCTS")
                  this.displayQWKAccts = (value == "TRUE");
               else if (settingUpper == "EXCLUDESYSOP")
                  this.excludeSysop = (value == "TRUE");
            }
            else if (settingsMode == "colors")
               this.colors[setting] = value;
         }
      }

      cfgFile.close();
   }
}
// For the DDUserLister class: Displays a User record in the list.
//
// Parameters:
//  pUser: A User object
function DDUserLister_DisplayUserInfo(pUser)
{
	if (pUser != null)
	{
		var lineText = this.colors["border"] + VERTICAL_SINGLE
				 + this.colors["userNum"] + centerText(format("%" + USER_NUM_LEN + "d", pUser.number), USER_NUM_LEN)
				 + this.colors["border"] + VERTICAL_SINGLE
				 + this.colors["userName"] + centerText(pUser.alias.substring(0, USER_NAME_LEN), USER_NAME_LEN)
				 + this.colors["border"] + VERTICAL_SINGLE
				 + this.colors["location"] + centerText(pUser.location.substring(0, LOCATION_LEN), LOCATION_LEN)
				 + this.colors["border"] + VERTICAL_SINGLE
				 + this.colors["lastCallDate"] + centerText(strftime("%m/%d/%y", pUser.stats.laston_date), LAST_CALL_DATE_LEN)
				 + this.colors["border"] + VERTICAL_SINGLE
				 + this.colors["connectionType"] + centerText(pUser.connection.substring(0, CONNECTION_TYPE_LEN), CONNECTION_TYPE_LEN)
				 + this.colors["border"] + VERTICAL_SINGLE
				 + this.colors["numCalls"] + centerText(format("%" + NUM_CALLS_LEN + "d", pUser.stats.total_logons), NUM_CALLS_LEN)
				 + this.colors["border"] + VERTICAL_SINGLE;
		console.center(lineText);
	}
}
// For the DDUserLister class: Lists all users.
function DDUserLister_ListAllusers() {
	// If gSortUserList is "ask", then ask the user if they want the list
	// sorted alphabetically.
	if ((typeof(this.sortUserList) == "string") && (this.sortUserList == "ask"))
	{
		console.crlf();
		// Note: If console.noyes() returns false, that means the
		// user chose Yes.
		this.sortUserList = !console.noyes("Sort alphabetically");
		if (this.sortUserList)
			this.sortType = "alpha";
	}

	// Clear the screen
	console.clear();

	// The following line which sets console.line_counter to 0 is a
	// kludge to disable Synchronet's automatic pausing after a
	// screenful of text.
	console.line_counter = 0;


	// Write the header
	console.print(this.colors["veryTopHeader"]);
	console.center("User List");
	// Upper border lines
	// User number section
	var lineText = this.colors["border"] + UPPER_LEFT_SINGLE;
	for (var i = 0; i < USER_NUM_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += T_SINGLE;
	// User name section
	for (var i = 0; i < USER_NAME_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += T_SINGLE;
	// User location section
	for (var i = 0; i < LOCATION_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += T_SINGLE;
	// Last call date section
	for (var i = 0; i < LAST_CALL_DATE_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += T_SINGLE;
	// Connection type section
	for (var i = 0; i < CONNECTION_TYPE_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += T_SINGLE;
	// # calls section
	for (var i = 0; i < NUM_CALLS_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += UPPER_RIGHT_SINGLE;
	console.center(lineText);
	// Column labels
	lineText = this.colors["border"] + VERTICAL_SINGLE + this.colors["colLabelText"]
				+ centerText("User#", USER_NUM_LEN) + this.colors["border"] + VERTICAL_SINGLE
				+ this.colors["colLabelText"] + centerText("User Name", USER_NAME_LEN) + this.colors["border"]
				+ VERTICAL_SINGLE + this.colors["colLabelText"]
				+ centerText("Location", LOCATION_LEN) + this.colors["border"]
				+ VERTICAL_SINGLE + this.colors["colLabelText"] + centerText("Last On", LAST_CALL_DATE_LEN)
				+ this.colors["border"] + VERTICAL_SINGLE + this.colors["colLabelText"]
				+ this.colors["colLabelText"] + centerText("Via", CONNECTION_TYPE_LEN)
				+ this.colors["border"] + VERTICAL_SINGLE + this.colors["colLabelText"]
				+ centerText("# calls", NUM_CALLS_LEN) + this.colors["border"] + VERTICAL_SINGLE;
	console.center(lineText);
	// Lower border lines for header
	// User number section
	var lineText = this.colors["border"] + LEFT_T_SINGLE;
	for (var i = 0; i < USER_NUM_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += CROSS_SINGLE;
	// User name section
	for (var i = 0; i < USER_NAME_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += CROSS_SINGLE;
	// User location section
	for (var i = 0; i < LOCATION_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += CROSS_SINGLE;
	// Last call date section
	for (var i = 0; i < LAST_CALL_DATE_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += CROSS_SINGLE;
	// Connection type section
	for (var i = 0; i < CONNECTION_TYPE_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += CROSS_SINGLE;
	// # calls section
	for (var i = 0; i < NUM_CALLS_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += RIGHT_T_SINGLE;
	console.center(lineText);

	// If the user wants the user list sorted, then display a
	// sorted list; otherwise, just go through and display the
	// users.
	if (this.sortUserList)
	{
		var userArray = getSortedUserList(!(this.excludeSysop), this.displayQWKAccts);
		for (var i = 0; i < userArray.length; ++i)
			this.displayUserInfo(userArray[i]);
	}
	else
	{
		// Determine the last user number
		var lastuser = 1;
		if (system.lastuser == undefined)	// v3.10
			lastuser = system.stats.total_users;
		else							// v3.11
			lastuser = system.lastuser;
		// List the users
		var theUser = null;
		var userArray = null;
		for (var userNum = 1; userNum <= lastuser; ++userNum)
		{
			// If the user's name is blank, then skip it.
			if (system.username(userNum).length == 0)
				continue;

			// If the sysop is to be excluded, then skip user # 1 (user #1
			// should be the sysop).
			if (this.excludeSysop && (userNum == 1))
				continue;

			// Load the user record
			theUser = new User(userNum);

			// If  this is a deleted or inactive user slot, then continue to
			// the next user.
			if (((theUser.settings & USER_DELETED) == USER_DELETED) || ((theUser.settings & USER_INACTIVE) == USER_INACTIVE))
				continue;

			// Skip this user if it's a guest account
			if (theUser.security.restrictions & UFLAG_G)
				continue;

			// Now, we should have a valid user account.

			// If this.displayQWKAccts is false and this is a QWK account, then
			// skip it.
			if (!(this.displayQWKAccts) && theUser.compare_ars("REST Q"))
				continue;

			// Display the user information.
			this.displayUserInfo(theUser);
		}
	}

	// Write the end border
	// User number section
	var lineText = this.colors["border"] + LOWER_LEFT_SINGLE;
	for (var i = 0; i < USER_NUM_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += BOTTOM_T_SINGLE;
	// User name section
	for (var i = 0; i < USER_NAME_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += BOTTOM_T_SINGLE;
	// User location section
	for (var i = 0; i < LOCATION_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += BOTTOM_T_SINGLE;
	// Last call date section
	for (var i = 0; i < LAST_CALL_DATE_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += BOTTOM_T_SINGLE;
	// Connection type section
	for (var i = 0; i < CONNECTION_TYPE_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += BOTTOM_T_SINGLE;
	// # calls section
	for (var i = 0; i < NUM_CALLS_LEN; ++i)
		lineText += HORIZONTAL_SINGLE;
	lineText += LOWER_RIGHT_SINGLE;
	console.center(lineText);
	console.print("hw" + system.stats.total_users + " btotal users.\r\n");
	// Display the system's pause/"hit a key" prompt
	console.print("p");
}
// For the DDUserLister class: Returns a string containing HTML to
// be displayed on the web.
//
// Parameters:
//  pTableClass: The class to use for the table.
function DDUserLister_GetHTML(pTableClass)
{
	var userListHTML = "";

	// If the user wants the user list sorted, then return a
	// sorted list; otherwise, display the users unsorted.
	if ((typeof(this.sortUserList) == "boolean") && this.sortUserList)
	{
		var userArray = getSortedUserList(!(this.excludeSysop), this.displayQWKAccts);
		if (userArray.length > 0)
		{
			userListHTML = "<table ";
			if ((pTableClass != null) && (typeof(pTableClass) != "undefined") && (pTableClass.length > 0))
				userListHTML += "class='" + pTableClass + "' ";
			// Columns: Alias, Location, Connection, Last On
			userListHTML += "cellspacing='1' cellpadding='2' summary='List of users for " + system.name + "'>\n"
								  + "<tr nowrap style='color: #ffffff; background-color: #0000bb; text-align: center;'>"
								  + "<th>Alias</th><th>Location</th><th>Connection</th><th>Last On</th></tr>\n";
			for (var i = 0; i < userArray.length; ++i)
			{
				// Append the row to the HTML.
				// Columns: Alias, Location, Connection, Last On
				userListHTML += "<tr nowrap style='background-color: #000000; text-align: center;'>"
								 // Show the user's handle with a link to their profile.
								 + "<td style='color: #00ffff;'><a href='/members/viewprofile.ssjs?showuser="
								 + userArray[i].number + "' style='color: #ffff00;'>" + userArray[i].alias + "</a></td>"
								 // Location
								 + "<td style='color: #ff00ff;'>" + userArray[i].location + "</td>"
								 // Connection
								 + "<td style='color: #0000ff;'>" + userArray[i].connection + "</td>"
								 // Last On
								 + "<td style='color: #ff0000; text-align: right;'>" + strftime("%Y-%m-%d", userArray[i].stats.laston_date) + "</td>"
								 + "</tr>\n";
			}
			userListHTML += "</table>\n";
		}
		else
			userListHTML = "<h3>There are currently no users on this system.</h3>";
	}
	else
	{
		// Determine the last user number
		var lastuser = 1;
		if (system.lastuser == undefined)	// v3.10
			lastuser = system.stats.total_users;
		else							// v3.11
			lastuser = system.lastuser + 1;
		if (lastuser > 0)
		{
			userListHTML = "<table ";
			if ((pTableClass != null) && (typeof(pTableClass) != "undefined") && (pTableClass.length > 0))
				userListHTML += "class='" + pTableClass + "' ";
			// Columns: Alias, Location, Connection, Last On
			userListHTML += "cellspacing='1' cellpadding='2' summary='List of users for " + system.name + "'>\n"
								  + "<tr nowrap style='color: #ffffff; background-color: #0000bb; text-align: center;'>"
								  + "<th>Alias</th><th>Location</th><th>Connection</th><th>Last On</th></tr>\n";

			var theUser = new User(1);
			for (var i = 1; i < lastuser; ++i)
			{
				// If this is the sysop and the option to exclude the sysop
				// is enabled, then skip this user.
				if ((i == 1) && this.excludeSysop)
					continue;

				theUser.number = i;

				// Skip this user if the user has been deleted or is inactive.
				if ((theUser.settings & USER_DELETED) || (theUser.settings & USER_INACTIVE))
					continue;
				// If this is a QWK account, skip it if the option to do so is enabled
				if (!(this.displayQWKAccts) && theUser.compare_ars("REST Q"))
					continue;
				// Skip this user if it's a guest account
				if (theUser.security.restrictions & UFLAG_G)
					continue;

				// Append the row to the HTML.
				// Columns: Alias, Location, Connection, Last On
				userListHTML += "<tr nowrap style='background-color: #000000; text-align: center;'>"
								 // Show the user's handle with a link to their profile.
								 + "<td style='color: #00ffff;'><a href='/members/viewprofile.ssjs?showuser="
								 + i + "' style='color: #ffff00;'>" + theUser.alias + "</a></td>"
								 // Location
								 + "<td style='color: #ff00ff;'>" + theUser.location + "</td>"
								 // Connection
								 + "<td style='color: #0000ff;'>" + theUser.connection + "</td>"
								 // Last On
								 + "<td style='color: #ff0000; text-align: right;'>" + strftime("%Y-%m-%d", theUser.stats.laston_date) + "</td>"
								 + "</tr>\n";
			}
			userListHTML += "</table>\n";
		}
		else
			userListHTML = "<h3>There are currently no users on this system.</h3>";
	}

	return userListHTML;
}

///////////////////////////////////////////////////////////////////////////////////
// General functions

// Centers some text within a specified field length.
//
// Parameters:
//  pText: The text to center
//  pFieldLen: The field length
function centerText(pText, pFieldLen)
{
	var centeredText = "";

   var textLen = strip_ctrl(pText).length;
	// If pFieldLen is less than the text length, then truncate the string.
	if (pFieldLen < textLen)
		centeredText = pText.substring(0, pFieldLen);
	else
	{
		// pFieldLen is at least equal to the text length, so we can
		// center the text.
		// Calculate the number of spaces needed to center the text.
		var numSpaces = pFieldLen - textLen;
		if (numSpaces > 0)
		{
			var rightSpaces = (numSpaces/2).toFixed(0);
			var leftSpaces = numSpaces - rightSpaces;
			// Build centeredText
			for (var i = 0; i < leftSpaces; ++i)
				centeredText += " ";
			centeredText += pText;
			for (var i = 0; i < rightSpaces; ++i)
				centeredText += " ";
		}
		else
		{
			// pFieldLength is the same length as the text.
			centeredText = pText;
		}
	}

	return centeredText;
}

// Removes multiple, leading, and/or trailing spaces
// The search & replace regular expressions used in this
// function came from the following URL:
//  http://qodo.co.uk/blog/javascript-trim-leading-and-trailing-spaces
//
// Parameters:
//  pString: The string to trim
//  pLeading: Whether or not to trim leading spaces (optional, defaults to true)
//  pMultiple: Whether or not to trim multiple spaces (optional, defaults to true)
//  pTrailing: Whether or not to trim trailing spaces (optional, defaults to true)
function trimSpaces(pString, pLeading, pMultiple, pTrailing)
{
	var leading = true;
	var multiple = true;
	var trailing = true;
	if(typeof(pLeading) != "undefined")
		leading = pLeading;
	if(typeof(pMultiple) != "undefined")
		multiple = pMultiple;
	if(typeof(pTrailing) != "undefined")
		trailing = pTrailing;

	// To remove both leading & trailing spaces:
	//pString = pString.replace(/(^\s*)|(\s*$)/gi,"");

	if (leading)
		pString = pString.replace(/(^\s*)/gi,"");
	if (multiple)
		pString = pString.replace(/[ ]{2,}/gi," ");
	if (trailing)
		pString = pString.replace(/(\s*$)/gi,"");

	return pString;
}

// Returns an array of the users on the BBS, sorted by username.
//
// Parameters:
//  pIncludeSysop: Whether or not to include the sysop
//  pIncludeQWKAccts: Whether or not to include QWK accounts
function getSortedUserList(pIncludeSysop, pIncludeQWKAccts)
{
	var userArray = new Array();

	// Determine the last user number
	var lastuser = 1;
	if (system.lastuser == undefined)	// v3.10
		lastuser = system.stats.total_users;
	else							// v3.11
		lastuser = system.lastuser;
	// Insert all the users into the array.
	var theUser = null;
	for (var userNum = 1; userNum <= lastuser; ++userNum)
	{
		// If the user's name is blank, then skip it.
		if (system.username(userNum).length == 0)
			continue;

		// If the sysop is to be excluded, then skip user # 1 (user #1
		// should be the sysop).
		if (!pIncludeSysop && (userNum == 1))
			continue;

		// Load the user record
		theUser = new User(userNum);

		// If  this is a deleted or inactive user slot, then don't
		// get this user.
		if (((theUser.settings & USER_DELETED) == USER_DELETED) ||
			 ((theUser.settings & USER_INACTIVE) == USER_INACTIVE))
		{
			continue;
		}

		// Skip this user if it's a guest account
		if (theUser.security.restrictions & UFLAG_G)
			continue;

		// Now, we should have a valid user account.

		// If this.displayQWKAccts is false and this is a QWK account, then
		// skip it.
		if (!pIncludeQWKAccts && theUser.compare_ars("REST Q"))
			continue;

		// Insert the user into the array
		userArray.push(theUser);
	}

	// Sort the array by user alias
	userArray.sort(function(pA, pB)
	{
		// Return -1, 0, or 1, depending on whether user A's alias
		// comes before, is equal to, or comes after user B's alias.
		var returnValue = 0;
		if (pA.alias < pB.alias)
			returnValue = -1;
		else if (pA.alias > pB.alias)
			returnValue = 1;
			
		return returnValue;
	});

	return userArray;
}